/* itick.c
 *
 * COPYRIGHT (c) 1995 AT&T Bell Laboratories.
 */

#include "ml-base.h"
#include "ml-c.h"
#include "cfun-proto-list.h"

/* _ml_RunT_itick : unit -> (int * int)
 *
 */
ml_val_t _ml_RunT_itick (ml_state_t *msp, ml_val_t arg)
{
    return RAISE_ERROR(msp, "itick unimplemented");

} /* end of _ml_RunT_itick */

