EXPORTS
    AddrOfCSymbol @1
    AddrTblApply @2
    AddrTblInsert @3
    AddrTblLookup @4
    AddrToCSymbol @5
    AllocGlobals @6
    AllocMLState @7
    ApplyMLFn @8
    ArenaName @9 DATA
    BIBOP @10 DATA
    BO_AddrToCodeObjTag @11
    BO_Alloc @12
    BO_AllocRegion @13
    BO_Free @14
    BO_GetCodeObjTag @15
    BO_GetDesc @16
    BindCFun @17
    BlastGC @18
    BlastGC_AssignLitAddrs @19
    BlastGC_BlastLits @20
    BlastGC_FinishUp @21
    BlastIn @22
    BlastOut @23
    BootML @24
    BuildLiterals @25
    CRoots @26 DATA
    ChooseSignal @27
    CmdLineArgs @28 DATA
    DebugF @29 DATA
    Die @30
    DisableProfSignals @31
    DumpObjectStrings @32 DATA
    EnableProfSignals @33
    Error @34
    Exit @35
    ExportCSymbol @36
    ExportFnImage @37
    ExportHeapImage @38
    ExportTableSz @39
    ExportedSymbols @40
    FPEEnable @41 DATA
    Flip @42
    FreeAddrTbl @43
    FreeExportTbl @44
    FreeGeneration @45
    GCMessages @46 DATA
    GetCPUTime @47
    GetObjGen @48
    GetSignalMask @49
    GetSignalState @50
    GetSzOption @51
    HeapIO_ReadBlock @52
    HeapIO_ReadExterns @53
    HeapIO_Seek @54
    HeapIO_WriteExterns @55
    HeapIO_WriteImageHeader @56
    ImportCSymbol @57
    ImportHeapImage @58
    InitCFunList @59
    InitFaultHandlers @60
    InitHeap @61
    InitMLState @62
    InitTimers @63
    InvokeGC @64
    InvokeGCWithRoots @65
    ListSignals @66
    LoadML @67
    LoadResumeState @68
    MEM_AllocMemObj @69
    MEM_FreeMemObj @70
    MEM_GetVMSize @71
    MEM_InitMemory @72
    MLCmdName @73 DATA
    ML_AllocArray @74
    ML_AllocBytearray @75
    ML_AllocCData @76
    ML_AllocCode @77
    ML_AllocRaw32 @78
    ML_AllocRaw64 @79
    ML_AllocRealdarray @80
    ML_AllocString @81
    ML_AllocVector @82
    ML_CData @83
    ML_CString @84
    ML_CStringList @85
    ML_ShrinkRaw32 @86
    ML_SysConst @87
    ML_SysConstList @88
    ML_X86Frame @89 DATA
    MajorGC @90
    MakeAddrTbl @91
    MakeHandlerArg @92
    MakeResumeCont @93
    MarkRegion @94
    MinorGC @95
    NeedGC @96
    NewDirtyVector @97
    NewExportTbl @98
    NewGeneration @99
    NumCRoots @100 DATA
    NumVProcs @101 DATA
    POSIX_Error_Library @102 DATA
    POSIX_FileSys_Library @103 DATA
    POSIX_IO_Library @104 DATA
    POSIX_OS_Library @105 DATA
    POSIX_ProcEnv_Library @106 DATA
    POSIX_Process_Library @107 DATA
    POSIX_Signal_Library @108 DATA
    POSIX_SysDB_Library @109 DATA
    POSIX_TTY_Library @110 DATA
    ParseHeapParams @111
    PauseUntilSignal @112
    ProfCntArray @113 DATA
    QualifyImageName @114
    RaiseMLExn @115
    RaiseSysError @116
    RawArgs @117 DATA
    RecordCSymbol @118
    RecordConcat @119
    RecordGlobals @120
    ResetTimers @121
    RestoreCState @122
    RunML @123
    RunTimeCompUnit @124 DATA
    SMLNJ_Date_Library @125 DATA
    SMLNJ_Math_Library @126 DATA
    SMLNJ_Prof_Library @127 DATA
    SMLNJ_RunT_Library @128 DATA
    SMLNJ_Sig_Library @129 DATA
    SMLNJ_Sock_Library @130 DATA
    SMLNJ_Time_Library @131 DATA
    SaveCState @132
    Say @133
    SayDebug @134
    ScanWeakPtrs @135
    SetSignalMask @136
    SetSignalState @137
    SilentLoad @138 DATA
    StartGCTimer @139
    StopGCTimer @140
    SwapBytes @141
    SysErr_id0 @142 DATA
    SysErr_s @143 DATA
    SystemRunML @144
    UNIX_Dynload_Library @145 DATA
    UnlimitedHeap @146 DATA
    VProc @147 DATA
    WR_OpenFile @148
    WR_OpenMem @149
    _ActiveProcs0 @150 DATA
    _Div_id0 @151 DATA
    _Div_s @152 DATA
    _ErrorNo @153 DATA
    _MLPollHandler @154 DATA
    _MLSignalHandler @155 DATA
    _ML_string0 @156 DATA
    _ML_string0_data @157 DATA
    _ML_vector0 @158 DATA
    _Overflow_id0 @159 DATA
    _Overflow_s @160 DATA
    _PervStruct @161 DATA
    _PollEvent0 @162 DATA
    _PollFreq0 @163 DATA
    _ProfCurrent @164 DATA
    _Sock_AddrFamily @165 DATA
    _Sock_Type @166 DATA
    _ml_Date_asctime @167
    _ml_Date_gmtime @168
    _ml_Date_localtime @169
    _ml_Date_mktime @170
    _ml_Date_strftime @171
    _ml_Math_atan64 @172
    _ml_Math_cos64 @173
    _ml_Math_ctlrndmode @174
    _ml_Math_exp64 @175
    _ml_Math_log64 @176
    _ml_Math_sin64 @177
    _ml_Math_sqrt64 @178
    _ml_NetDB_gethostbyaddr @179
    _ml_NetDB_gethostbyname @180
    _ml_NetDB_gethostname @181
    _ml_NetDB_getnetbyaddr @182
    _ml_NetDB_getnetbyname @183
    _ml_NetDB_getprotbyname @184
    _ml_NetDB_getprotbynum @185
    _ml_NetDB_getservbyname @186
    _ml_NetDB_getservbyport @187
    _ml_OS_poll @188
    _ml_OS_tmpname @189
    _ml_P_Error_errmsg @190
    _ml_P_Error_geterror @191
    _ml_P_Error_listerrors @192
    _ml_P_FileSys_access @193
    _ml_P_FileSys_chdir @194
    _ml_P_FileSys_chmod @195
    _ml_P_FileSys_chown @196
    _ml_P_FileSys_closedir @197
    _ml_P_FileSys_fchmod @198
    _ml_P_FileSys_fchown @199
    _ml_P_FileSys_fpathconf @200
    _ml_P_FileSys_fstat @201
    _ml_P_FileSys_fstat_64 @202
    _ml_P_FileSys_ftruncate @203
    _ml_P_FileSys_ftruncate_64 @204
    _ml_P_FileSys_getcwd @205
    _ml_P_FileSys_link @206
    _ml_P_FileSys_lstat @207
    _ml_P_FileSys_lstat_64 @208
    _ml_P_FileSys_mkdir @209
    _ml_P_FileSys_mkfifo @210
    _ml_P_FileSys_opendir @211
    _ml_P_FileSys_openf @212
    _ml_P_FileSys_osval @213
    _ml_P_FileSys_pathconf @214
    _ml_P_FileSys_readdir @215
    _ml_P_FileSys_readlink @216
    _ml_P_FileSys_rename @217
    _ml_P_FileSys_rewinddir @218
    _ml_P_FileSys_rmdir @219
    _ml_P_FileSys_stat @220
    _ml_P_FileSys_stat_64 @221
    _ml_P_FileSys_symlink @222
    _ml_P_FileSys_umask @223
    _ml_P_FileSys_unlink @224
    _ml_P_FileSys_utime @225
    _ml_P_IO_close @226
    _ml_P_IO_dup @227
    _ml_P_IO_dup2 @228
    _ml_P_IO_fcntl_d @229
    _ml_P_IO_fcntl_gfd @230
    _ml_P_IO_fcntl_gfl @231
    _ml_P_IO_fcntl_l @232
    _ml_P_IO_fcntl_l_64 @233
    _ml_P_IO_fcntl_sfd @234
    _ml_P_IO_fcntl_sfl @235
    _ml_P_IO_fsync @236
    _ml_P_IO_lseek @237
    _ml_P_IO_lseek_64 @238
    _ml_P_IO_osval @239
    _ml_P_IO_pipe @240
    _ml_P_IO_read @241
    _ml_P_IO_readbuf @242
    _ml_P_IO_write @243
    _ml_P_IO_writebuf @244
    _ml_P_ProcEnv_ctermid @245
    _ml_P_ProcEnv_environ @246
    _ml_P_ProcEnv_getegid @247
    _ml_P_ProcEnv_getenv @248
    _ml_P_ProcEnv_geteuid @249
    _ml_P_ProcEnv_getgid @250
    _ml_P_ProcEnv_getgroups @251
    _ml_P_ProcEnv_getlogin @252
    _ml_P_ProcEnv_getpgrp @253
    _ml_P_ProcEnv_getpid @254
    _ml_P_ProcEnv_getppid @255
    _ml_P_ProcEnv_getuid @256
    _ml_P_ProcEnv_isatty @257
    _ml_P_ProcEnv_setgid @258
    _ml_P_ProcEnv_setpgid @259
    _ml_P_ProcEnv_setsid @260
    _ml_P_ProcEnv_setuid @261
    _ml_P_ProcEnv_sysconf @262
    _ml_P_ProcEnv_time @263
    _ml_P_ProcEnv_times @264
    _ml_P_ProcEnv_ttyname @265
    _ml_P_ProcEnv_uname @266
    _ml_P_Process_alarm @267
    _ml_P_Process_exec @268
    _ml_P_Process_exece @269
    _ml_P_Process_execp @270
    _ml_P_Process_exit @271
    _ml_P_Process_fork @272
    _ml_P_Process_kill @273
    _ml_P_Process_osval @274
    _ml_P_Process_pause @275
    _ml_P_Process_sleep @276
    _ml_P_Process_waitpid @277
    _ml_P_Signal_osval @278
    _ml_P_SysDB_getgrgid @279
    _ml_P_SysDB_getgrnam @280
    _ml_P_SysDB_getpwnam @281
    _ml_P_SysDB_getpwuid @282
    _ml_P_TTY_osval @283
    _ml_P_TTY_tcdrain @284
    _ml_P_TTY_tcflow @285
    _ml_P_TTY_tcflush @286
    _ml_P_TTY_tcgetattr @287
    _ml_P_TTY_tcgetpgrp @288
    _ml_P_TTY_tcsendbreak @289
    _ml_P_TTY_tcsetattr @290
    _ml_P_TTY_tcsetpgrp @291
    _ml_Proc_argv @292
    _ml_Proc_cmd_name @293
    _ml_Proc_raw_argv @294
    _ml_Prof_getpquantum @295
    _ml_Prof_setpref @296
    _ml_Prof_setptimer @297
    _ml_RunT_alloc_code @298
    _ml_RunT_blast_in @299
    _ml_RunT_blast_out @300
    _ml_RunT_debug @301
    _ml_RunT_dummy @302
    _ml_RunT_export_fun @303
    _ml_RunT_export_heap @304
    _ml_RunT_gc_ctl @305
    _ml_RunT_itick @306
    _ml_RunT_mkexec @307
    _ml_RunT_mkliterals @308
    _ml_RunT_record1 @309
    _ml_RunT_recordconcat @310
    _ml_RunT_setitimer @311
    _ml_RunT_sysinfo @312
    _ml_Sig_getsigmask @313
    _ml_Sig_getsigstate @314
    _ml_Sig_listsigs @315
    _ml_Sig_pause @316
    _ml_Sig_setsigmask @317
    _ml_Sig_setsigstate @318
    _ml_Sock_accept @319
    _ml_Sock_bind @320
    _ml_Sock_close @321
    _ml_Sock_connect @322
    _ml_Sock_ctlBROADCAST @323
    _ml_Sock_ctlDEBUG @324
    _ml_Sock_ctlDONTROUTE @325
    _ml_Sock_ctlKEEPALIVE @326
    _ml_Sock_ctlLINGER @327
    _ml_Sock_ctlNODELAY @328
    _ml_Sock_ctlOOBINLINE @329
    _ml_Sock_ctlRCVBUF @330
    _ml_Sock_ctlREUSEADDR @331
    _ml_Sock_ctlSNDBUF @332
    _ml_Sock_frominetaddr @333
    _ml_Sock_fromunixaddr @334
    _ml_Sock_getATMARK @335
    _ml_Sock_getERROR @336
    _ml_Sock_getNREAD @337
    _ml_Sock_getTYPE @338
    _ml_Sock_getaddrfamily @339
    _ml_Sock_getpeername @340
    _ml_Sock_getsockname @341
    _ml_Sock_inetany @342
    _ml_Sock_listaddrfamilies @343
    _ml_Sock_listen @344
    _ml_Sock_listsocktypes @345
    _ml_Sock_recv @346
    _ml_Sock_recvbuf @347
    _ml_Sock_recvbuffrom @348
    _ml_Sock_recvfrom @349
    _ml_Sock_sendbuf @350
    _ml_Sock_sendbufto @351
    _ml_Sock_setNBIO @352
    _ml_Sock_shutdown @353
    _ml_Sock_socket @354
    _ml_Sock_socketpair @355
    _ml_Sock_toinetaddr @356
    _ml_Sock_tounixaddr @357
    _ml_Time_gettime @358
    _ml_Time_timeofday @359
    _ml_U_Dynload_dlclose @360
    _ml_U_Dynload_dlerror @361
    _ml_U_Dynload_dlopen @362
    _ml_U_Dynload_dlsym @363
    _ml_posix_nv_lookup @364
    _util_NetDB_mkhostent @365
    _util_NetDB_mknetent @366
    _util_NetDB_mkservent @367
    _util_Sock_ControlFlg @368
    array_a @369 DATA
    array_v @370 DATA
    bind_cfun_a @371 DATA
    bind_cfun_v @372 DATA
    build_literals_a @373 DATA
    callc_a @374 DATA
    callc_v @375 DATA
    create_b_a @376 DATA
    create_b_v @377 DATA
    create_r_a @378 DATA
    create_r_v @379 DATA
    create_s_a @380 DATA
    create_s_v @381 DATA
    create_v_a @382 DATA
    create_v_v @383 DATA
    fegetround @384 DATA
    fesetround @385 DATA
    floor_a @386 DATA
    floor_v @387 DATA
    handle_a @388 DATA
    handle_v @389 DATA
    init_fn @390
    isRuntimeOption @391
    logb_a @392 DATA
    logb_v @393 DATA
    machine_id @394 DATA
    main @395
    pollh_resume @396 DATA
    pollh_return_a @397 DATA
    pollh_return_c @398 DATA
    request_fault @399 DATA
    restoreregs @400 DATA
    return_a @401 DATA
    return_c @402 DATA
    saveregs @403 DATA
    scalb_a @404 DATA
    scalb_v @405 DATA
    set_request @406 DATA
    sigh_resume @407 DATA
    sigh_return_a @408 DATA
    sigh_return_c @409 DATA
    try_lock_a @410 DATA
    try_lock_v @411 DATA
    unlock_a @412 DATA
    unlock_v @413 DATA
